@@ -1,4 +1,4 @@
-__version__ = '1.4.1'
+__version__ = '1.5.0'
 
 from .utils import *
 from .source import *
@@ -6,22 +6,28 @@ from .block_source import *
 from .likelihood import *
 from .inference import *
 
-
-# LXe model construction
+# Original flamedisx models
+# Accessible under fd root package (for now), for backwards compatibility
 from .lxe_blocks.energy_spectrum import *
 from .lxe_blocks.quanta_generation import *
 from .lxe_blocks.quanta_splitting import *
 from .lxe_blocks.detection import *
 from .lxe_blocks.double_pe import *
 from .lxe_blocks.final_signals import *
-
 from .lxe_sources import *
 
-
-# XENON specifics
+# XENON(1T) models
+# Accessible under fd root package (for now), for backwards compatibility
 from .xenon.resource import *
 from .xenon.itp_map import *
 from .xenon.data import *
-
 from .xenon.x1t_sr0 import *
 from .xenon.x1t_sr1 import *
+
+# NEST models
+# Access through fd.nest.xxx
+from . import nest
+
+# LUX models
+# Access through fd.lux.xxx
+from . import lux
