@@ -16,20 +16,74 @@ class Block:
 
     For example, P(electrons_detected | electrons_produced).
     """
+    #: Source the block belongs to
+    source: fd.Source = None
+
+    #: Names of dimensions of the block's compute result
     dimensions: ty.Tuple[str]
 
-    depends_on: ty.Tuple[str] = tuple()
+    #: Additional dimensions used in the block computation.
+    #: Label True if they represent an internally contracted hidden variable;
+    #: these will be added to inner_dimensions so domain tensors are calculated
+    #: automatically.
+    #: Label False otherwise; these will be added to bonus_dimensions. Thus,
+    #: any additional domain tensors utilising them will need calculating via
+    #: the block overriding _domain_dict_bonus())
+    extra_dimensions: ty.Tuple[ty.Tuple[str, bool]]
 
+    #: Blocks whose result this block expects as an extra keyword
+    #: argument to compute. Specify as ((block_dims, argument_name), ...),
+    #: where block_dims is the dimensions-tuple of the block, and argument_name
+    #: the expected name of the compute keyword argument.
+    depends_on: ty.Tuple[ty.Tuple[ty.Tuple[str], str]] = tuple()
+
+    #: Names of model functions defined in this block
     model_functions: ty.Tuple[str] = tuple()
+
+    #: Names of model functions that take an additional first argument
+    #: ('bonus arg') defined in this block; must be a subset of model_functions
     special_model_functions: ty.Tuple[str] = tuple()
+
+    #: Names of columns this block expects to be array-valued
     array_columns: ty.Tuple[str] = tuple()
+
+    #: Frozen model functions defined in this block
     frozen_model_functions: ty.Tuple[str] = tuple()
+
+    #: Additional attributes this Block will furnish the source with.
+    #: These can be overriden by Source attributes, just like model functions.
     model_attributes: ty.Tuple[str] = tuple()
 
     def __init__(self, source):
         self.source = source
         assert len(self.dimensions) in (1, 2), \
             "Blocks must output 1 or 2 dimensions"
+        # Currently only support 1 extra_dimension per block
+        assert len(self.extra_dimensions) <= 1, \
+            f"{self} has >1 extra dimension!"
+
+    # Redirect all model attribute queries to the linked source,
+    # as soon as one exists, and has the relevant attribute.
+    # We need __getattribute__, not __getattr__, to catch access
+    # to attributes that exist.
+    def __getattribute__(self, name):
+        # Can't use self.something here without recursion...
+        linked_attributes = (
+            super().__getattribute__('model_functions')
+            + super().__getattribute__('model_attributes'))
+        source = super().__getattribute__('source')
+
+        if (name in linked_attributes and hasattr(source, name)):
+            return getattr(source, name)
+        else:
+            return super().__getattribute__(name)
+
+    def __setattr__(self, name, value):
+        if (name in (self.model_functions + self.model_attributes)
+                and self.source is not None):
+            setattr(self.source, name, value)
+        else:
+            super().__setattr__(name, value)
 
     def setup(self):
         """Do any necessary initialization.
@@ -47,7 +101,16 @@ class Block:
         return self.source.gimme_numpy(*args, **kwargs)
 
     def compute(self, data_tensor, ptensor, **kwargs):
-        kwargs.update(self.source._domain_dict(self.dimensions, data_tensor))
+        if len(self.extra_dimensions) == 0:
+            kwargs.update(self.source._domain_dict(
+                self.dimensions, data_tensor))
+        else:
+            if self.extra_dimensions[0][1] is True:
+                raise NotImplementedError
+            else:
+                kwargs.update(self.source._domain_dict(
+                    self.dimensions, data_tensor))
+                kwargs.update(self._domain_dict_bonus(data_tensor))
         result = self._compute(data_tensor, ptensor, **kwargs)
         assert result.dtype == fd.float_type(), \
             f"{self}._compute returned tensor of wrong dtype!"
@@ -100,6 +163,17 @@ class Block:
         """Add _min and _max for each dimension to d in-place"""
         raise NotImplementedError
 
+    def _domain_dict_bonus(self, d):
+        """Calculate any additional intenal tensors arising from the use of
+        bonus_dimensions in a block. Override within block"""
+        raise NotImplementedError
+
+    def _calculate_dimsizes_special(self):
+        """Re-calculate dimension size and steps differently for any
+        dimensions; will need to override _calculate_dimsizes_special()
+        within a block"""
+        pass
+
 
 @export
 class FirstBlock(Block):
@@ -131,10 +205,17 @@ class BlockModelSource(fd.Source):
     """Source whose model is split over different Blocks
     """
 
+    #: Blocks the source is built from.
+    #: simulate will be called from first to last, annotate from last to first.
     model_blocks: tuple
-    final_dimensions: tuple
+
+    #: Dimensions provided by the first block
     initial_dimensions: tuple
 
+    #: Additional dimensions used in the block computation;
+    #: see Block.extra_dimensions for info
+    extra_dimensions = ()
+
     def __init__(self, *args, **kwargs):
         if isinstance(self.model_blocks[0], FirstBlock):
             # Blocks have already been instantiated
@@ -148,6 +229,7 @@ class BlockModelSource(fd.Source):
         # Collect attributes from the different blocks in this dictionary:
         collected = {k: [] for k in (
             'dimensions',
+            'extra_dimensions',
             'model_functions',
             'special_model_functions',
             'model_attributes',
@@ -164,29 +246,20 @@ class BlockModelSource(fd.Source):
                     raise ValueError(
                         f"{k} in {b} should be a tuple, not a {type(k)}")
 
-            attributes = set(b.model_functions
-                             + b.special_model_functions
-                             + b.model_attributes)
-
-            for x in attributes:
-                # If a source attribute was specified,
-                # override the block's attribute.
-                if hasattr(self, x):
-                    setattr(b, x, getattr(self, x))
-
-            # Now that the block is properly furnished with all attributes,
-            # call the block setup.
+            # Call the setup method. This method is not really needed anymore;
+            # blocks can simply override __init__ for setup, as long as they
+            # call super().__init__ *first* (else self.source would not be set)
             b.setup()
 
             # Set the source attributes from the block's attributes.
-            # If we did this before b.setup(), the source and block attributes
-            # could diverge, causing potential confusion.
-            for x in attributes:
+            # From here on, block attributes become invisible and irrelevant,
+            # since get/setattr on the blocks will redirect to the source.
+            for x in set(b.model_functions + b.model_attributes):
                 setattr(self, x, getattr(b, x))
 
             # Collect all information from the block.
-            # We also do this after the setup; the array columns field in
-            # particular is nice to change in the block setup
+            # We do this after the setup; the array columns field in
+            # particular is nice to change in the block setup code
             for k in collected:
                 collected[k] += getattr(b, k)
 
@@ -216,13 +289,17 @@ class BlockModelSource(fd.Source):
             if k == 'dimensions':
                 # Dimensions is a special case, see below
                 continue
-            setattr(self, k, tuple(set(v)))
+            setattr(self, k, getattr(self, k) + tuple(set(v)))
 
-        self.inner_dimensions = tuple([
-            d for d in collected['dimensions']
-            if ((d not in self.final_dimensions)
-                and (d not in self.model_blocks[0].dimensions))])
+        self.inner_dimensions = tuple(
+            [d for d in collected['dimensions']
+                if ((d not in self.final_dimensions)
+                    and (d not in self.model_blocks[0].dimensions))]
+            + [d[0] for d in collected['extra_dimensions']
+                if d[1] is True])
         self.initial_dimensions = self.model_blocks[0].dimensions
+        self.bonus_dimensions = tuple([
+            d[0] for d in collected['extra_dimensions'] if d[1] is False])
 
         super().__init__(*args, **kwargs)
 
@@ -230,9 +307,8 @@ class BlockModelSource(fd.Source):
     def _find_block(blocks,
                     has_dim: ty.Union[list, tuple, set],
                     exclude: Block = None):
-        """Find a block with a dimension in allowed
-        Return (dimensions, b), or
-         raises BlockNotFoundError if no such block found.
+        """Find a block with a dimension in has_dim, other than the block in
+        exclude. Return (dimensions, b), or raises BlockNotFoundError.
         """
         for dims, b in blocks.items():
             if b is exclude:
@@ -244,6 +320,7 @@ class BlockModelSource(fd.Source):
 
     def _differential_rate(self, data_tensor, ptensor):
         results = {}
+        already_stepped = ()  # Avoid double-multiplying to account for stepping
 
         for b in self.model_blocks:
             b_dims = b.dimensions
@@ -259,7 +336,22 @@ class BlockModelSource(fd.Source):
                 kwargs.update(self._domain_dict(dependency_dims, data_tensor))
 
             # Compute the block
-            results[b_dims] = r = b.compute(data_tensor, ptensor, **kwargs)
+            r = b.compute(data_tensor, ptensor, **kwargs)
+
+            # Scale the block by stepped dimensions, if not already done in
+            # another block
+            for dim in b_dims:
+                if (dim in self.inner_dimensions) and \
+                        (dim not in self.no_step_dimensions) and \
+                        (dim not in already_stepped):
+                    steps = self._fetch(dim+'_steps', data_tensor=data_tensor)
+                    step_mul = tf.repeat(steps[:, o], tf.shape(r)[1], axis=1)
+                    step_mul = tf.repeat(step_mul[:, :, o],
+                                         tf.shape(r)[2], axis=2)
+                    r *= step_mul
+                    already_stepped += (dim,)
+
+            results[b_dims] = r
 
             # Try to matrix multiply with earlier blocks, until we cannot
             # do so anymore.
@@ -395,6 +487,12 @@ class BlockModelSource(fd.Source):
     def add_derived_observables(self, d):
         pass
 
+    def calculate_dimsizes_special(self):
+        """Custom calulcation of any dimension sizes and steps; override
+        _calculate_dimsizes_special within block"""
+        for b in self.model_blocks:
+            b._calculate_dimsizes_special()
+
 
 class BlockNotFoundError(Exception):
     pass
