@@ -51,7 +51,8 @@ class LogLikelihood:
                 ty.Dict[str, pd.DataFrame]] = None,
             free_rates=None,
             batch_size=10,
-            max_sigma=3,
+            max_sigma=None,
+            max_dim_size=120,
             n_trials=int(1e5),
             log_constraint=None,
             bounds_specified=True,
@@ -76,6 +77,10 @@ class LogLikelihood:
 
         :param max_sigma: Maximum sigma to use in bounds estimation.
             Higher numbers give better accuracy, at the cost of performance.
+            If not specified, each source will use its own default.
+
+        :param max_dim_size: Maximum bounds size for inner_dimensions,
+            excluding no_step_dimensions
 
         :param n_trials: Number of Monte-Carlo trials for mu estimation.
 
@@ -133,6 +138,7 @@ class LogLikelihood:
         self.sources = {
             sname: sclass(data=None,
                           max_sigma=max_sigma,
+                          max_dim_size=max_dim_size,
                           # The source will filter out parameters it does not
                           # take
                           fit_params=list(k for k in common_param_specs.keys()),
@@ -497,22 +503,22 @@ class LogLikelihood:
         """Return best-fit parameter dict
 
         :param guess: Guess parameters: dict {param: guess} of guesses to use.
-        Any omitted parameters will be guessed at LogLikelihood.defaults()
+            Any omitted parameters will be guessed at LogLikelihood.defaults()
         :param fix: dict {param: value} of parameters to keep fixed
-        during the minimzation.
+            during the minimzation.
         :param optimizer: 'tf', 'minuit' or 'scipy'
         :param get_lowlevel_result: Returns the full optimizer result instead
-        of the best fit parameters. Bool.
+            of the best fit parameters. Bool.
         :param get_history: Returns the history of optimizer calls instead
-        of the best fit parameters. Bool.
+            of the best fit parameters. Bool.
         :param use_hessian: If True, uses flamedisxs' exact Hessian
-        in the optimizer. Otherwise, most optimizers estimate it by finite-
-        difference calculations.
+            in the optimizer. Otherwise, most optimizers estimate it by finite-
+            difference calculations.
         :param return_errors: If using the minuit minimizer, instead return
-        a 2-tuple of (bestfit dict, error dict).
-        In case optimizer is minuit, you can also pass 'hesse' or 'minos' here.
+            a 2-tuple of (bestfit dict, error dict).
+            If the optimizer is minuit, you can also pass 'hesse' or 'minos'.
         :param allow_failure: If True, raise a warning instead of an exception
-        if there is an optimizer failure.
+            if there is an optimizer failure.
         """
         if bounds is None:
             bounds = dict()
@@ -600,36 +606,36 @@ class LogLikelihood:
             use_hessian=True,
             allow_failure=False,
     ):
-        """Return frequentist limit or confidence interval
+        """Return frequentist limit or confidence interval.
+
+        Returns a float (for upper or lower limits) or a 2-tuple of floats
+        (for a central interval)
 
         :param parameter: string, the parameter to set the interval on
         :param bestfit: {parameter: value} dictionary, global best-fit.
-        If omitted, will compute it using bestfit.
+            If omitted, will compute it using bestfit.
         :param guess: {param: value} guess of the result, or None.
-        If omitted, nuisance parameters will be guessed equal to bestfit.
-        If omitted, guess for target parameters will be based on asymptotic
-        parabolic computation.
+            If omitted, nuisance parameters will be guessed equal to bestfit.
+            If omitted, guess for target parameters will be based on asymptotic
+            parabolic computation.
         :param fix: {param: value} to fix during interval computation.
-        Result is only valid if same parameters were fixed for bestfit.
+            Result is only valid if same parameters were fixed for bestfit.
         :param confidence_level: Requried confidence level of the interval
         :param kind: Type of interval, 'upper', 'lower' or 'central'
         :param sigma_guess: Guess for one sigma uncertainty on the target
-        parameter. If not provided, will be computed from Hessian.
+            parameter. If not provided, will be computed from Hessian.
         :param t_ppf: returns critical value as function of parameter
-        Use Wilks' theorem if omitted.
+            Use Wilks' theorem if omitted.
         :param t_ppf_grad: return derivative of t_ppf
         :param t_ppf_hess: return second derivative of t_ppf
         :param tilt_overshoot: Set tilt so the limit's log likelihood will
-        overshoot the target value by roughly this much.
+            overshoot the target value by roughly this much.
         :param optimizer_kwargs: dict of additional arguments for optimizer
         :param allow_failure: If True, raise a warning instead of an exception
-        if there is an optimizer failure.
+            if there is an optimizer failure.
         :param use_hessian: If True, uses flamedisxs' exact Hessian
-        in the optimizer. Otherwise, most optimizers estimate it by finite-
-        difference calculations.
-
-        Returns a float (for upper or lower limits)
-        or a 2-tuple of floats (for a central interval)
+            in the optimizer. Otherwise, most optimizers estimate it by finite-
+            difference calculations.
         """
         if optimizer_kwargs is None:
             optimizer_kwargs = dict()
