@@ -1,6 +1,8 @@
 from copy import copy
 from contextlib import contextmanager
 import inspect
+import typing as ty
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -17,19 +19,52 @@ o = tf.newaxis
 
 @export
 class Source:
+    #: Number of event batches to use in differential rate computations
     n_batches = None
+
+    #: Number of fake events that were padded to the final batch
+    #: to make it match the batch size
     n_padding = None
+
+    #: Whether to trace (compile into a tensorflow graph) the differential
+    #: rate computation
     trace_difrate = True
+    default_max_sigma = 3
 
-    model_functions = tuple()
-    special_model_functions = tuple()
-    inner_dimensions = tuple()
+    #: Names of model functions
+    model_functions: ty.Tuple[str] = tuple()
 
-    frozen_model_functions = tuple()
-    array_columns = tuple()
+    #: Names of model functions that take an additional first argument
+    #: ('bonus arg'). This must be a subset of model_functions.
+    special_model_functions: ty.Tuple[str] = tuple()
 
-    # Final observable dimensions; for use in domain / cross-domain
-    final_dimensions = tuple()
+    #: Model functions whose results should be evaluated once per event,
+    #: then stored with the data. For example, non-tensorflow functions.
+    #: Note these cannot have any fittable parameters.
+    frozen_model_functions: ty.Tuple[str] = tuple()
+
+    #: Names of final observable dimensions (e.g. s1, s2)
+    #: for use in domain / cross-domain
+    final_dimensions: ty.Tuple[str] = tuple()
+
+    #: Names of dimensions of hidden variables (e.g. produced electrons)
+    #: for which domain computations and dimsize calculations are to be done
+    inner_dimensions: ty.Tuple[str] = tuple()
+
+    #: inner_dimensions excluded from variable stepping logic, i.e.
+    #: for which the domain is always a single interval of integers
+    no_step_dimensions: ty.Tuple[str] = tuple()
+
+    #: Names of dimensions of hidden variables for which
+    #: dimsize calculations are NOT done here (but in user-defined code)
+    #: but for which we DO track _min and _dimsizes
+    bonus_dimensions: ty.Tuple[str] = tuple()
+
+    #: Names of array-valued data columns
+    array_columns: ty.Tuple[str] = tuple()
+
+    #: Any additional source attributes that should be configurable.
+    model_attributes = tuple()
 
     # List all columns that are manually _fetch ed here
     # These will be added to the data_tensor even when the model function
@@ -37,7 +72,8 @@ class Source:
     def extra_needed_columns(self):
         return []
 
-    data = None
+    #: The fully annotated event data
+    data: pd.DataFrame = None
 
     ##
     # Initialization and helpers
@@ -61,6 +97,10 @@ class Source:
             for pname, p in inspect.signature(f).parameters.items():
                 if pname == 'self':
                     continue
+                if pname in self.model_functions:
+                    raise AttributeError(
+                        f"{pname} is used both as a model function and "
+                        f"as a parameter of a model function, {fname}")
                 if p.default is inspect.Parameter.empty:
                     if fname in self.special_model_functions and not seen_special:
                         seen_special = True
@@ -75,10 +115,71 @@ class Source:
                     defaults[pname] = tf.convert_to_tensor(
                         p.default, dtype=fd.float_type())
 
+    def print_config(self,
+                     format='table',
+                     column_widths=(40, 20),
+                     omit=tuple()):
+        """Print the defaults of all parameters (from Source.defaults), and of
+        model functions that have been set to constants (from Source.f_dims)
+
+        :param format: 'table' to print a fixed-width table, 'config' to print
+            as a configuration file
+        :param column_widths: 2-tuple of column widths to use for table format
+        :param omit: settings to omit from printout. Useful for format='config',
+            since some things (like arrays and tensors) cannot be evaluated
+            from their string representation.
+        """
+
+        def print_row(*cols, header=False):
+            cols = [str(x).replace('\n', '') for x in cols]
+            if format == 'table':
+                print(''.join([
+                    col.ljust(w) if len(col) < w else col[:w - 3] + '...'
+                    for col, w in zip(cols, column_widths)]))
+            else:
+                result = '# ' if header else ''
+                result += ' = '.join(cols)
+                print(result)
+
+        format_value = str if format == 'table' else repr
+
+        def print_line(marker='-'):
+            if format == 'table':
+                print(marker * sum(column_widths))
+            else:
+                print()
+
+        print_row('Parameter', 'Default', header=True)
+        print_line()
+        for pname, default in sorted(self.defaults.items()):
+            if pname in omit:
+                continue
+            print_row(pname, format_value(default.numpy()))
+        print()
+
+        print_row("Constant (could be made a function)", 'Default', header=True)
+        print_line()
+        for fname in sorted(self.model_functions):
+            if fname in omit:
+                continue
+            f = getattr(self, fname)
+            if not callable(f):
+                print_row(fname, format_value(f))
+        print()
+
+        print_row("Other attribute", 'Default', header=True)
+        print_line()
+        for fname in sorted(self.model_attributes):
+            if fname in omit:
+                continue
+            print_row(fname, format_value(getattr(self, fname)))
+        print()
+
     def __init__(self,
                  data=None,
                  batch_size=10,
-                 max_sigma=3,
+                 max_sigma=None,
+                 max_dim_size=120,
                  data_is_annotated=False,
                  _skip_tf_init=False,
                  _skip_bounds_computation=False,
@@ -90,21 +191,34 @@ class Source:
         :param data: Dataframe with events to use in the inference
         :param batch_size: Number of events / tensorflow batch
         :param max_sigma: Hint for hidden variable bounds computation
+            If omitted, set to default_max_sigma
+        :param max_dim_size: Maximum bounds size for inner_dimensions,
+            excluding no_step_dimensions
         :param data_is_annotated: If True, skip annotation
         :param _skip_tf_init: If True, skip tensorflow cache initialization
         :param _skip_bounds_computation: If True, skip bounds compuation
         :param fit_params: List of parameters to fit
         :param progress: whether to show progress bars for mu estimation
-          (if data is not None)
-        :param params: New defaults to use
+            (if data is not None)
+        :param params: New defaults to for parameters, and new values for
+        constant-valued model functions.
         """
+        if max_sigma is None:
+            max_sigma = self.default_max_sigma
         self.max_sigma = max_sigma
+        self.max_dim_size = max_dim_size
 
         # Check for duplicated model functions
         for attrname in ['model_functions', 'special_model_functions']:
             l_ = getattr(self, attrname)
             if len(set(l_)) != len(l_):
                 raise ValueError(f"{attrname} contains duplicates: {l_}")
+        # Check all special model functions are actually model functions
+        for fname in self.special_model_functions:
+            if fname not in self.model_functions:
+                raise ValueError(
+                    f"{attrname} is listed as a special model function, "
+                    f"but not as a model function")
 
         # Discover which functions need which arguments / dimensions
         # Discover possible parameters.
@@ -119,6 +233,13 @@ class Source:
         ctc += self.extra_needed_columns()                  # Manually fetched columns
         ctc += self.frozen_model_functions                     # Frozen methods (e.g. not tf-compatible)
         ctc += [x + '_min' for x in self.inner_dimensions]  # Left bounds of domains
+        ctc += [x + '_max' for x in self.inner_dimensions]  # Right bounds of domains
+        ctc += [x + '_min' for x in self.bonus_dimensions]  # Left bounds of domains
+        ctc += [x + '_steps' for x in self.inner_dimensions]  # Step sizes
+        ctc += [x + '_steps' for x in self.bonus_dimensions]  # Step sizes
+        ctc += [x + '_dimsizes' for x in self.inner_dimensions]  # Dimension sizes
+        ctc += [x + '_dimsizes' for x in self.bonus_dimensions]  # Dimension sizes
+        ctc += [x + '_dimsizes' for x in self.final_dimensions]  # Dimension sizes
         ctc = list(set(ctc))
 
         self.column_index = fd.index_lookup_dict(ctc,
@@ -155,11 +276,33 @@ class Source:
         if not _skip_tf_init:
             self.trace_differential_rate()
 
-    def set_defaults(self, **params):
+    def set_defaults(self, *, config=None, **params):
+        # Load new params from configuration files
+        params = {**fd.load_config(config), **params}
+
+        # Apply new defaults
+        unused = dict()
         for k, v in params.items():
             if k in self.defaults:
+                # Change a default
                 self.defaults[k] = tf.convert_to_tensor(
                     v, dtype=fd.float_type())
+            elif k in self.model_functions:
+                # Change a model function, only allowed if it is a constant
+                # (otherwise, just subclass the source)
+                f = getattr(self, k)
+                if callable(f):
+                    raise AttributeError(
+                        f"Use source subclassing to override the non-constant "
+                        f"model function {k}")
+                setattr(self, k, v)
+            elif k in self.model_attributes:
+                # Change a generic model attribute
+                setattr(self, k, v)
+            else:
+                unused[k] = v
+        if unused:
+            warnings.warn(f"Defaults for unused settings ignored: {unused}")
 
     def set_data(self,
                  data=None,
@@ -196,11 +339,11 @@ class Source:
             self.add_extra_columns(self.data)
             if not _skip_bounds_computation:
                 self._annotate()
+                self._calculate_dimsizes(self.max_dim_size)
 
         if not _skip_tf_init:
             self._check_data()
             self._populate_tensor_cache()
-            self._calculate_dimsizes()
 
     def _check_data(self):
         """Do any final checks on the self.data dataframe,
@@ -241,14 +384,43 @@ class Source:
             result,
             [self.n_batches, -1, self.n_columns_in_data_tensor])
 
-    def _calculate_dimsizes(self):
+    def cap_dimsizes(self, dim, cap):
+        if dim in self.no_step_dimensions:
+            pass
+        else:
+            self.dimsizes[dim] = cap * np.greater(self.dimsizes[dim], cap) + \
+                self.dimsizes[dim] * np.less_equal(self.dimsizes[dim], cap)
+
+    def _calculate_dimsizes(self, max_dim_size):
         self.dimsizes = dict()
+        d = self.data
         for dim in self.inner_dimensions:
-            ma = self._fetch(dim + '_max')
-            mi = self._fetch(dim + '_min')
-            self.dimsizes[dim] = int(tf.reduce_max(ma - mi + 1).numpy())
+            ma = d[dim + '_max'].to_numpy()
+            mi = d[dim + '_min'].to_numpy()
+            self.dimsizes[dim] = ma - mi + 1
+            # Ensure we don't go over max_dim_size in a domain
+            self.cap_dimsizes(dim, max_dim_size)
+
+            # Calculate steps if we have cappeed the dimesize
+            steps = tf.where((ma-mi+1) > self.dimsizes[dim],
+                             tf.math.ceil((ma-mi) / (self.dimsizes[dim]-1)),
+                             1).numpy()  # Cover to at least the upper bound
+            # Store the steps in the dataframe
+            d[dim + '_steps'] = steps
+
         for dim in self.final_dimensions:
-            self.dimsizes[dim] = 1
+            self.dimsizes[dim] = np.ones(len(d))
+
+        # Calculate all custom dimsizes
+        self.calculate_dimsizes_special()
+
+        # Store the dimsizes in the dataframe
+        for dim in self.inner_dimensions:
+            d[dim + "_dimsizes"] = self.dimsizes[dim]
+        for dim in self.bonus_dimensions:
+            d[dim + "_dimsizes"] = self.dimsizes[dim]
+        for dim in self.final_dimensions:
+            d[dim + "_dimsizes"] = self.dimsizes[dim]
 
     @contextmanager
     def _set_temporarily(self, data, **kwargs):
@@ -315,13 +487,13 @@ class Source:
         :param fname: Name of the model function to compute
         :param bonus_arg: If fname takes a bonus argument, the data for it
         :param numpy_out: If True, return (tuple of) numpy arrays,
-        otherwise (tuple of) tensors.
+            otherwise (tuple of) tensors.
         :param data_tensor: Data tensor, columns as self.column_index
-        If not given, use self.data (used in annotate)
+            If not given, use self.data (used in annotate)
         :param ptensor: Parameter tensor, columns as self.param_id
-        If not give, use defaults dictionary (used in annotate)
-        Before using gimme, you must use set_data to
-        populate the internal caches.
+            If not given, use defaults dictionary (used in annotate)
+            Before using gimme, you must use set_data to
+            populate the internal caches.
         """
         assert (bonus_arg is not None) == (fname in self.special_model_functions)
         assert isinstance(fname, str), \
@@ -393,6 +565,7 @@ class Source:
         return [self.batch_size, self.n_columns_in_data_tensor]
 
     def trace_differential_rate(self):
+        """Compile the differential rate computation to a tensorflow graph"""
         input_signature = (
             tf.TensorSpec(shape=self._batch_data_tensor_shape(),
                           dtype=fd.float_type()),
@@ -420,7 +593,7 @@ class Source:
     ##
 
     def domain(self, x, data_tensor=None):
-        """Return (n_events, |possible x values|) matrix containing all
+        """Return (n_events, n_x) matrix containing all
         possible integer values of x for each event.
 
         If x is a final dimension (e.g. s1, s2), we return an (n_events, 1)
@@ -429,20 +602,21 @@ class Source:
         if x in self.final_dimensions:
             return self._fetch(x, data_tensor=data_tensor)[:, o]
 
-        x_range = tf.cast(tf.range(self.dimsizes[x]),
-                          dtype=fd.float_type())[o, :]
+        # Cover the bounds range in integer steps not necessarily of 1
         left_bound = self._fetch(x + '_min', data_tensor=data_tensor)[:, o]
+        steps = self._fetch(x + '_steps', data_tensor=data_tensor)[:, o]
+        x_range = tf.range(tf.reduce_max(self._fetch(x + '_dimsizes', data_tensor=data_tensor))) * steps
         return left_bound + x_range
 
     def cross_domains(self, x, y, data_tensor):
-        """Return (x, y) two-tuple of (n_events, |x|, |y|) tensors
+        """Return (x, y) two-tuple of (n_events, n_x, n_y) tensors
         containing possible integer values of x and y, respectively.
         """
         # TODO: somehow mask unnecessary elements and save computation time
         x_domain = self.domain(x, data_tensor)
         y_domain = self.domain(y, data_tensor)
-        result_x = tf.repeat(x_domain[:, :, o], y_domain.shape[1], axis=2)
-        result_y = tf.repeat(y_domain[:, o, :], x_domain.shape[1], axis=1)
+        result_x = tf.repeat(x_domain[:, :, o], tf.shape(y_domain)[1], axis=2)
+        result_y = tf.repeat(y_domain[:, o, :], tf.shape(x_domain)[1], axis=1)
         return result_x, result_y
 
     ##
@@ -487,7 +661,7 @@ class Source:
 
         Careful: ensure mutual constraints are accounted for first!
         (e.g. fixing energy for a modulating WIMP has consequences for the
-         time distribution.)
+        time distribution.)
         """
         if fix_truth is not None:
             for k, v in fix_truth.items():
@@ -596,7 +770,11 @@ class ColumnSource(Source):
     """Source that expects precomputed differential rate in a column,
     and precomputed mu in an attribute
     """
+
+    #: Name of the data column containing the precomputed differential rate
     column = 'rename_me!'
+
+    #: Expected events for this source
     mu = 42.
 
     def extra_needed_columns(self):
@@ -614,3 +792,6 @@ class ColumnSource(Source):
     def random_truth(self, n_events, fix_truth=None, **params):
         print(f"{self.__class__.__name__} cannot generate events, skipping")
         return pd.DataFrame()
+
+    def calculate_dimsizes_special(self):
+        pass
