@@ -1,3 +1,4 @@
+from pathlib import Path
 import subprocess
 
 import numpy as np
@@ -53,15 +54,17 @@ def lookup_axis1(x, indices, fill_value=0):
        returning fill_value for out-of-range indices.
     """
     # Save shape of x and flatten
-    ind_shape = indices.shape
-    a, b = x.shape
+    ind_shape = tf.shape(indices)
+    a = tf.shape(x)[0]
+    b = tf.shape(x)[1]
     x = tf.reshape(x, [-1])
 
-    legal_index = indices < b
+    legal_index = tf.cast(indices, dtype=int_type()) < b
 
     # Convert indices to legal indices in flat array
-    indices = tf.clip_by_value(indices, 0., b - 1.)
-    indices = indices + b * tf.range(a, dtype=float_type())[:, o, o]
+    indices = tf.clip_by_value(indices, 0., tf.cast(b, dtype=float_type())-1.)
+    indices = indices + tf.cast(b, dtype=float_type()) * \
+        tf.range(a, dtype=float_type())[:, o, o]
     indices = tf.reshape(indices, shape=(-1,))
     indices = tf.dtypes.cast(indices, dtype=int_type())
 
@@ -201,7 +204,7 @@ def index_lookup_dict(names, column_widths=None):
      (tf.constant integers.)
 
     :param column_widths: dictionary mapping names to column width.
-    For columns with width > 1, the result contains a tensor slice.
+        For columns with width > 1, the result contains a tensor slice.
     """
     names = list(names)
     if column_widths is None:
@@ -249,3 +252,30 @@ def run_command(command):
             stderr=subprocess.STDOUT) as p:
         for line in iter(p.stdout.readline, ''):
             print(line.rstrip())
+
+
+@export
+def load_config(config_files=None):
+    """Return dictionary of configuration options from (a) python file(s)"""
+    if config_files is None:
+        return {}
+    if isinstance(config_files, str):
+        # Support one or multiple files
+        config_files = (config_files,)
+
+    config = dict()
+    for filename in config_files:
+        if not filename.endswith('.py'):
+            # This is (hopefully) a named config shipped with flamedisx
+            filename = Path(__file__).parent / 'configs' / (filename + '.py')
+
+        # Adapted from https://stackoverflow.com/a/37611448
+        with open(filename) as f:
+            code = compile(f.read(), filename, 'exec')
+        captured_locals = dict()
+        exec(code, globals(), captured_locals)
+        config.update({
+            k: v for k, v in captured_locals.items()
+            if not k.startswith('_')})
+
+    return config
